import time
from machine import I2C

# i2c address
LPS22HB_I2C_ADDRESS = 0x5C

# Register
LPS22HB_ID = 0xB1
# Interrupt register
LPS22HB_INT_CFG = 0x0B
# Pressure threshold registers
LPS22HB_THS_P_L = 0x0C
LPS22HB_THS_P_H = 0x0D
# Who am I
LPS22HB_WHO_AM_I = 0x0F
# Control registers
LPS22HB_CTRL_REG1 = 0x10
LPS22HB_CTRL_REG2 = 0x11
LPS22HB_CTRL_REG3 = 0x12
# FIFO configuration register
LPS22HB_FIFO_CTRL = 0x14
# Reference pressure registers
LPS22HB_REF_P_XL = 0x15
LPS22HB_REF_P_L = 0x16
LPS22HB_REF_P_H = 0x17
# Pressure offset registers
LPS22HB_RPDS_L = 0x18
LPS22HB_RPDS_H = 0x19
# Resolution register
LPS22HB_RES_CONF = 0x1A
# Interrupt register
LPS22HB_INT_SOURCE = 0x25
# FIFO status register
LPS22HB_FIFO_STATUS = 0x26
# Status register
LPS22HB_STATUS = 0x27
# Pressure output registers
LPS22HB_PRESS_OUT_XL = 0x28
LPS22HB_PRESS_OUT_L = 0x29
LPS22HB_PRESS_OUT_H = 0x2A
# Temperature output registers
LPS22HB_TEMP_OUT_L = 0x2B
LPS22HB_TEMP_OUT_H = 0x2C
# Filter reset register
LPS22HB_RES = 0x33

# Conversion factors units of measurement
CELSIUS_TO_KELVIN = 273.15;
MILLIBARS_TO_PASCALS = 100;
MILLIBARS_TO_MILLIMETERSHG = 0.75;

class LPS22HB(object):
    def __init__(self, address=LPS22HB_I2C_ADDRESS):
        self._address = address
        self._bus = I2C(1)
        # Wait for reset to complete
        self._reset()
        data = self._read_byte(LPS22HB_CTRL_REG1)
        data |= 1 << 6
        data |= 1 << 4
        self._write_byte(LPS22HB_CTRL_REG1, data)

    def _reset(self):
        data = self._read_byte(LPS22HB_CTRL_REG2)
        data |= 0x04
        # SWRESET Set 1
        self._write_byte(LPS22HB_CTRL_REG2, data)
        while data:
            data = self._read_byte(LPS22HB_CTRL_REG2)
            data &= 0x04

    def LPS22HB_START_ONESHOT(self):
        data = self._read_byte(LPS22HB_CTRL_REG2)
        # ONE_SHOT Set 1
        data |= 0x01
        self._write_byte(LPS22HB_CTRL_REG2, data)

    def _read_byte(self, cmd):
        rec = self._bus.readfrom_mem(int(self._address), int(cmd), 1)
        return rec[0]

    def _read_byte16(self, cmd):
        LSB = self._bus.readfrom_mem(int(self._address), int(cmd), 1)
        MSB = self._bus.readfrom_mem(int(self._address), int(cmd)+1, 1)
        return (MSB[0] << 8) + LSB[0]

    def _write_byte(self, cmd, val):
        self._bus.writeto_mem(int(self._address), int(cmd), bytes([int(val)]))

    def read_temperature_c(self):
        data = [0, 0]
        data[0] = self._read_byte(LPS22HB_TEMP_OUT_L)
        data[1] = self._read_byte(LPS22HB_TEMP_OUT_H)
        return ((data[1] << 8)+data[0])/100.0

    def read_pressure_millibars(self):
        data = [0, 0, 0]
        data[0] = self._read_byte(LPS22HB_PRESS_OUT_XL)
        data[1] = self._read_byte(LPS22HB_PRESS_OUT_L)
        data[2] = self._read_byte(LPS22HB_PRESS_OUT_H)
        return ((data[2] << 16)+(data[1] << 8) + data[0]) / 4096.0
    
    def read_pressure_millimeters_hg(self):
        return self.read_pressure_millibars() * MILLIBARS_TO_MILLIMETERSHG
    
    def read_altitude(self):
        return (1 - ((self.read_pressure_millibars() / 1013.25) ** 0.190263)) * 44330.8;